extern char* time_message_pointer;
/*
 * 	
 * ***** the dallas driver stuff *****	
 * 
 */
float get_temp(unsigned char type) { 
  /* Type is DEGREES_F or DEGREES_C for the returned float */
  /* see if the sensor is reported to be working */		
  switch (dal_dat.sensor_ok) {	     
  case (TEMP_VALID): {		
    /* only get and convert temp if the sensor is found */
    /* get the raw_temperature  */		
    dal_dat.raw_temperature = read_dallas(dal_dat.protocol,9);
    dal_dat.count_per_c = read_dallas(0xa9,9);
    /* get the slope */
    dal_dat.count_remain = read_dallas(0xa0,9);
    /* and the remainder of the count */
    if ((dal_dat.raw_temperature & 0x0100)) {
      /* Is negative, so mask, invert, and bump */
      dal_dat.minus = 1;
      dal_dat.holding = (dal_dat.raw_temperature & 0x00ff);
      dal_dat.holding = (dal_dat.holding ^ 0xff);
      dal_dat.holding = dal_dat.holding + 1;	
    }
    
    else {
      /* a positive temp, no need to alter */ 
      dal_dat.holding = (dal_dat.raw_temperature & 0x00ff);
      dal_dat.minus = 0;
    }
    /* convert 0.5 deg_c steps to deg c */			
    dal_dat.result = (dal_dat.holding / 2.00);			    
    /* force the math to make sense, if the count_remain */
    /* is > count_per_c ! */
    if (dal_dat.count_remain > dal_dat.count_per_c) {
      dal_dat.count_remain = (dal_dat.count_remain - dal_dat.count_per_c);
    }
	
    dal_dat.result = ((dal_dat.result - 0.25) + \
		      ((dal_dat.count_per_c - dal_dat.count_remain) 
		       / dal_dat.count_per_c));
    
    if (dal_dat.minus) {
      /* convert to real neg floating point number */
      dal_dat.result = (-dal_dat.result);
    }

    /* handle any offset */
    dal_dat.result = (dal_dat.result + dal_dat.offset);
    dal_dat.degrees_c = dal_dat.result;	  /* store the results */		
    /* convert to deg_f */
    dal_dat.degrees_f = (((9.0/5.0) * dal_dat.result) + 32.0);
    dal_dat.result = dal_dat.holding = 0; /* reset temp data */	

    /* do the min/max temps */
    if (dal_dat.degrees_c < dal_dat.min_c) {
      dal_dat.min_c = dal_dat.degrees_c;
      strcpy((unsigned char*)&dal_dat.last_min_time,
	     (unsigned char*)time_message_pointer);
    }

    if (dal_dat.degrees_f < dal_dat.min_f) {
      dal_dat.min_f = dal_dat.degrees_f;
      strcpy((unsigned char*)&dal_dat.last_min_time,
	     (unsigned char*)time_message_pointer);
    }

    if (dal_dat.degrees_c > dal_dat.max_c) {
      dal_dat.max_c = dal_dat.degrees_c;
      strcpy((unsigned char*)&dal_dat.last_max_time,
	     (unsigned char*)time_message_pointer);
    }

    if (dal_dat.degrees_f > dal_dat.max_f) {
      dal_dat.max_f = dal_dat.degrees_f;
      strcpy((unsigned char*)&dal_dat.last_max_time,
	     (unsigned char*)time_message_pointer);
    }

    if (type == DEGREES_C) {		  
      /* return deg_c */
      return(dal_dat.degrees_c);
    }
    
    return(dal_dat.degrees_f);		  
    /* type is DEGREES_F if we got here */
  }
  case (TEMP_INVALID):
	/* temp sensor was not OK when this function was called! */
	return((float)TEMP_NOT_VALID);	  
  } /* end switch (dal_dat.sensor_ok... */
}   /* end float get_temp(... */

/*
 * 
 * ***** The dallas control primitives *****
 * 
 */
void send_dallas(unsigned long protocol, unsigned long data){
  /* protocol is the 8 bit protocol to send
     data is the 8 bit data to send */
  dallas_reset();
  send_dallas_primitive(protocol,8);
  send_dallas_primitive(data,8);
  dallas_reset();
}

void send_dallas_primitive(unsigned long data, unsigned char count) {
  /* data is the data to send, and count is the number of bits to send */
  unsigned char bit_count = 0;
  unsigned char control_port_state;
  unsigned char data_bit_out = 0;
  control_port_state = get_port_control();
  control_port_state = (control_port_state & 0x0f);
  for (bit_count = 1;bit_count <= count;bit_count = bit_count +1) {
    /* set the strobe bit */ 
    set_port_control(control_port_state | 0x01); 
    /* make sure the ~rst bit is hi! */
    data_bit_out = ((data & 0x01)| 0x02); 
    /* send the bit */
    set_port_data(data_bit_out);
    /* shift by a bit */
    data = (data >> 1);
    /* reset the strobe bit */
    control_port_state = (control_port_state  & 0xfe);
    set_port_control(control_port_state);
  }
}

unsigned long read_dallas(unsigned long protocol, unsigned char bits) {
  unsigned long storage = 0x0000;
  unsigned long temp_storage = 0x0000;
  unsigned char control_port_state;
  unsigned char bit_count = 0;
  dallas_reset();
  send_dallas_primitive(protocol,8);
  /* set port to input mode */
  control_port_state = get_port_control();
  control_port_state = (control_port_state | 0x20);
  set_port_control(control_port_state);
  
  for (bit_count = 1;bit_count <= bits;bit_count = bit_count +1) {
    /* set the strobe bit */
    control_port_state = (control_port_state | 0x21); 
    set_port_control(control_port_state);
    /* get the bit & mask D0 */
    temp_storage = get_port_data();		     
    temp_storage = (temp_storage & 0x01);
    if (temp_storage == 0x01) {
      storage = (storage | 0x8000);
    }
    storage = (storage >> 1); 
    /* reset the strobe bit */
    control_port_state = (control_port_state & 0xfe); 
    set_port_control(control_port_state);
  }
  
  control_port_state = (control_port_state & 0xdf);
  /* set port to output mode */
  set_port_control(control_port_state);    
  dallas_reset();
  /* shift the LSB into place */
  storage = (storage >> ((16 - bits) - 1));
  return(storage);
}

void dallas_reset(void) {
  set_port_data(0x00);	/* take d1 low */
  set_port_data(0x02);	/* and back hi */
}

void dallas_init(void) {
  float junk_data;
  /* init a dallas device, if there */
  send_dallas(0x000c,0x000a);		
  /* start temp conv */
  send_dallas_primitive(0xee,0x08); 
  dal_dat.stat = read_dallas(0xac,8);		
  if ((dal_dat.stat != 0x00) & 
      (dal_dat.stat != 0xff)) {
    dal_dat.sensor_ok = TEMP_VALID;	
    /* if stat looks like the sensor is connected */		
  }				        
  /* report if we found it or not */
  syslog_temp_stat();			

  /* If the sensor is there, init the min/max values */
  if (dal_dat.sensor_ok) {
    /* do a couple of reads to kick things off */
    junk_data = get_temp(DEGREES_F);    
    junk_data = get_temp(DEGREES_F);
    /* Init all the min/max junk... */
    dal_dat.min_c = dal_dat.degrees_c;  
    dal_dat.max_c = dal_dat.degrees_c;
    dal_dat.min_f = dal_dat.degrees_f;
    dal_dat.max_f = dal_dat.degrees_f;
    strcpy((unsigned char*)&dal_dat.last_min_time,
	   (unsigned char*)time_message_pointer);
    strcpy((unsigned char*)&dal_dat.last_max_time,
	   (unsigned char*)time_message_pointer);
  }
}
