/*
 * 	
 * ***** FIFO stuff... *****
 * 
 */
int init_fifo(void) {
  int result = 0;
  int i = 0;
  mode_t mode = 0666;
  /* Mark the FIFO not init'ed yet */
  fifo.init_ok = 0; 
  strcpy((char*)&fifo.in_pipe_name,(char*)&in_str);
  /* look for the pipes */
  result = open(&fifo.in_pipe_name,
		O_RDONLY | O_NONBLOCK | O_NOCTTY);

  if (result < 0) {
    /* It's not there, so create it ! */
    sprintf(syslog_string_ptr,\
	    "Creating input FIFO %s \n",fifo.in_pipe_name);
    syslog_send(syslog_string_ptr);	
    
    result = mkfifo(&fifo.in_pipe_name,mode);
    if (result < 0) {
      /* couldn't make the fifo, and that's bad! */
      sprintf(syslog_string_ptr,\
	      "Couldn't create input FIFO. %i\n",result);
      syslog_send(syslog_string_ptr);	
      return(-1);
    }
  }
  
  fifo.input_fd = open(&fifo.in_pipe_name,
		       (O_RDONLY | O_NONBLOCK | O_NOCTTY));
  /* input pipe is there and open, store the information */
  /* mark it open in the structure */
  /* Then, syslog it */
  sprintf(syslog_string_ptr,\
	  "Input FIFO open _FD_ %i \n",fifo.input_fd);
  syslog_send(syslog_string_ptr);	
  
  if (fifo.input_fd > 0) {
    fifo.init_ok = 1;
    fifo.update = 0;
    fifo.p_input_buf_base = (unsigned char*)&fifo.input;
    fifo.p_input_buf_offset = (unsigned char*)&fifo.input;
    /* init the input buf pointers */
    for (i = 0; i<40; i++){
      fifo.line_0_buf[i] = 0x20;
      fifo.line_1_buf[i] = 0x20;
    }
    /* setup the line buffers */
    fifo.line_0_buf[40] = 0x00;
    fifo.line_1_buf[40] = 0x00;
    fifo.line_0_ready = 0x00;
    fifo.line_1_ready = 0x00;
  }
  return(0);
}

int read_fifo() {
  /* Read the data from the FIFO, and place the */
  /* extracted strings into buffers for line_0 and line_1 */
  /* such that they can be displayed on the LCD */
  /* Filter non-printing chars, as well as limit the */
  /* length of the strings to be displayed (so it will not */
  /* overflow the string space with long input data) */
  /* read up to 256 chars from the input buf */
  int num_chars = 0;
  int disp_offset = 0;			
  int i = 0;			
  int line_0_chars = 0;			
  num_chars = read(fifo.input_fd,
		   fifo.p_input_buf_base, 
		   256);
  if (num_chars == 0) {
    /* nothing in the buffer */
    return(-1);
  }

  /* Check for an error... */				       
  if ((num_chars == -1) & (errno == EAGAIN)) {
    return(-1);			      
  }
		
  if (num_chars == -1) {
    sprintf(syslog_string_ptr,
	    "Error %i reading the input FIFO.\n",
	    errno); 
    /* got some error reading the pipe */
    syslog_send(syslog_string_ptr);
    return(-1);
  }
  
  if (num_chars > 80) {
    /* limit the size of the data for the two lines */
    num_chars = 80;
  }				      
  
  /* ok, there's something in the buffer, and there are no errors */
  disp_offset = 0;
  for (i=0; i<num_chars; i++) {		
    /* strip chars < 0x20 | > 0x7e , and 40 chars / line max*/
    if ((fifo.input[i] >= 0x20) & (fifo.input[i] <= 0x7e)) {
      fifo.line_0_buf[disp_offset] = fifo.input[i];
      disp_offset++;		        
      /* if there are less than 40 chars, I stop at the \n */
    }			                
    /* and the same for line 1. Lines _MUST_ end with a \n */
    if ((fifo.input[i] == '\n') | (i > 39)) break;
  }
  
  if (fifo.input[i] != '\n') {		
    /* if it was not a \n that ended the display line, */
    while(1) {			        
      /* continue looking until we find one... */
      i++;		                
      /* the display line may have ended due to length... */
      if (fifo.input[i] == '\n') break;
    }
  }
  
  line_0_chars = i;			
  /* line_0_chars tells up how many chars to the first \n */
  /* in the buffer. */
  fifo.line_0_buf[disp_offset] = 0x00;
  fifo.line_0_ready = 0x01;            
  /* string 0 is done..... */
  i++;					
  /* point to the next string */
  
  disp_offset = 0;
  for (i=i; i<num_chars; i++) {
    if ((fifo.input[i] >= 0x20) & (fifo.input[i] <= 0x7e)) {
      fifo.line_1_buf[disp_offset] = fifo.input[i];
      disp_offset++;
    }
    if ((fifo.input[i] == '\n') | ((i - line_0_chars) > 39)) break;
  }
  fifo.line_1_buf[disp_offset] = 0x00;
  fifo.line_1_ready = 0x01;		
  /* string 1 is done..... */
  
  /* I always read two lines of data from the buffer */
  /* we need more error checking and handling here later */
  /* to handle race conditions, etc */
  /* normal exit for now... */
  return(0);			 
}

int display_fifo_buf() {
  /* exit if not ready to write to display */
  if (!fifo.line_0_ready & !fifo.line_1_ready) return(-1);
  /* home the display */
  display_address(0x00);		
  display_string((unsigned char*)&fifo.line_0_buf);
  /* cursor to the second line..... */
  line_2();				
  display_string((unsigned char*)&fifo.line_1_buf);
  /* We just updated the display, reset the counter */
  fifo.update = 0;			
  /* and clear the 'line_ready' flags */
  fifo.line_0_ready = 0x00;	
  fifo.line_1_ready = 0x00;
  return(0);
}

int check_and_display_fifo() {
  int status = read_fifo();
  if (status) {			
    /* was not a 'good' read */	
    /* bump the 'not-updated' counter */
    fifo.update++;			
    return(-1);
  }
  /* read was good, now display */
  status = display_fifo_buf();		
  return(status);
}
