/*
 * 
 *  ***** IO Port manip *****
 * 
 */
 
/* get the port space from the kernel */
void get_port(unsigned long base_addr, unsigned long port_len) {
  if (ioperm(base_addr,port_len,1) < 0) {
    sprintf(syslog_string_ptr,
	    "Can't get ioperm for 0x%3x-0x%3x. Exiting.\n",
	    base_addr,
	    (base_addr+port_len));
    syslog_send(syslog_string_ptr);
    sprintf(syslog_string_ptr,
	    "Are you root?\n");
    syslog_send(syslog_string_ptr);
    fflush();
    exit (-1);
  }
				
  if (ioperm(DUMMY_OP_PORT,1,1) < 0) {
    sprintf(syslog_string_ptr,
	    "Can't get ioperm for 0x%3x. Exiting.\n"
	    ,DUMMY_OP_PORT);
    syslog_send(syslog_string_ptr);
    sprintf(syslog_string_ptr,
	    "Are you root?\n");
    syslog_send(syslog_string_ptr);
    fflush();
    exit (-1);
  }
  
}

void clean_up(int my_signal) {
  /* release IO ports when done... */
  unsigned char signal_string[10];
  unsigned char exit_message[41] = "    Display Driver\0"; 
  /* Clear the display on exit */
  switch (my_signal) {
  case SIGINT:
    strcpy((unsigned char*)&signal_string,
	   "    SIGINT\0");
    break;
  case SIGTERM:
    strcpy((unsigned char*)&signal_string,
	   "    SIGTERM\0");
    break;
  default:
    strcpy((unsigned char*)&signal_string,
	   "    UNKNOWN\0");
  }
	  
  display_address(0x00);        
  /* This makes use of the autofill mode */
  display_string(exit_message); 
  /* display the exit string */
  line_2();
  /* now, display the second line of the exit string, indicating */
  /* the signal that caused us to exit */
  sprintf((unsigned char*)&exit_message,
	  "%s Shutdown",
	  signal_string);
  display_string(exit_message);
  /* Enter a message in the syslog about the SIGNALED SHUTDOWN */
  sprintf(syslog_string_ptr,
	  "%s Driver Shutdown\n",
	  &signal_string);
  syslog_send(syslog_string_ptr);
  fflush();

  if (ioperm(DUMMY_OP_PORT,1,0) < 0) {
    sprintf(syslog_string_ptr,
	    "Can't release ioperm for 0x%3x\n",
	    DUMMY_OP_PORT);
    syslog_send(syslog_string_ptr);	
    fflush();
    exit (-1);
  }

  if (ioperm(port_base_addr,port_len,0) < 0) {
    sprintf(syslog_string_ptr,
	    "Can't release ioperm for 0x%3x-0x%3x\n",
	    k_port_base,
	    (k_port_base+port_len));
    syslog_send(syslog_string_ptr);	
    fflush();
    exit (-1);
  }

  if (fifo.input_fd & fifo.init_ok) {
    if (close(fifo.in_pipe_name) < 0) {
      sprintf(syslog_string_ptr,
	      "Can't close input FIFO\n");
      syslog_send(syslog_string_ptr);			
      fflush();
    };
  }
  exit(0); /* Done */
}

/*
 * 
 *  ***** IO Port read and write primitives *****
 * 
 */ 
/* write data to the port */
void set_port_data(unsigned char dat) {
  unsigned short data = (k_port_base);
  outb_p(dat,data);
}

/* get the data from the status port */
unsigned char get_port_status() {
  unsigned short status = (k_port_base + 1);
  return inb_p(status);
}

/* get the data from the control port */
unsigned char get_port_control() {
  unsigned short control = (k_port_base + 2);
  return inb_p(control);
}

/* get data from the port */
unsigned char get_port_data() {
  unsigned short data = (k_port_base);
  return inb_p(data);
}

/* write the data to the status port */
void set_port_status(unsigned char dat) {
  unsigned short status = (k_port_base + 1);
  outb_p(dat,status);
}

/* write the data to the control port */
void set_port_control(unsigned char dat) {
  unsigned short control = (k_port_base + 2);
  outb_p(dat,control);
}

/* send a char to the display */
void display_data(unsigned char data) {
  unsigned char holding;
  holding = 0x02;
  set_port_control(holding);
  set_port_data(data);
  write_enable_bit();
  /* update pos pointers */
  disp_cp.cur_current_pos++;
  disp_cp.cur_remain_chars--;	  
  delay_done();
}

/* display_address() sets the char pos for the display */
void display_address(unsigned char address) {
  unsigned char holding;
  /* update pos. pointer */
  disp_cp.cur_current_pos = address; 
  
  if (address < 0x40) disp_cp.cur_line = 0;
  else disp_cp.cur_line = 1;
  
  switch (disp_size) {
  case e_20x2:
    disp_cp.cur_remain_chars = (20 - (address & 0x3f));
    break;
    
  case e_24x2:
    disp_cp.cur_remain_chars = (24 - (address & 0x3f));
    break;
  } /* end switch (disp_size)... */
  
  holding = 0x0a;
  /* set the hi bit to control the display address */
  address = (address | 0x80); 
  set_port_control(holding);
  set_port_data(address);
  write_enable_bit();
  delay_done();
}

/* toggle the enable bit of the LCD display */
/* (for writing a char/command) */
void write_enable_bit() {
  unsigned char holding;
  holding = get_port_control();
  holding = (holding | 0x04);
  set_port_control(holding);
  outb_p(0x00,DUMMY_OP_PORT);
  holding = (holding & 0xfb);
  set_port_control(holding);
}

/* send a control command to the LCD display */	
void display_control(unsigned char control) {
  unsigned char holding;
  holding = 0x0a;
  set_port_control(holding);
  set_port_data(control);
  write_enable_bit(); /* wiggle bit */
  delay_done();
}

unsigned char display_read(unsigned char mask) {
  unsigned char holding;
  unsigned char data;
  /* read the port control byte */
  holding = get_port_control();
  /* only look at bits 3-1 */
  holding = (holding & 0x0e);	
  /* set port into read mode and display into read mode */
  /* (bit 5 hi, and bit 1 low...) */ 
  holding = ((holding | 0x20) & 0xfd);	
  set_port_control(holding);
  /* set the enable bit */
  holding = (holding | 0x04);
  set_port_control(holding);
  /* wait for a uS (or two) for the port to turn around */
  /* with a dummy write */
  outb_p(0x00,DUMMY_OP_PORT);
  outb_p(0x00,DUMMY_OP_PORT);
  /* read the data */
  data = get_port_data();
  /* reset the enable bit */
  holding = (holding & 0xfb);
  /* return port to write mode */	
  holding = (holding | 0x02);
  set_port_control(holding);
  /* wait for a uS (or two) for the port to turn around */
  /* with a dummy write */
  outb_p(0x00,DUMMY_OP_PORT);
  outb_p(0x00,DUMMY_OP_PORT);
  data = (data & mask);
  return (data);	
  /*
   * You may think that there are a lot of dummy writes in there, but it makes
   * the display work with a 25ft (max len) cable. Look into this later and see
   * if we want to remove the long cable restriction
   */	
}

void delay_done() {
  /* see if the display is done with command/data */
  unsigned char temp;
  unsigned long loop_counter = 0;
  do	{
    /* read and mask */
    temp = display_read(BUSY_READ_MASK);
    loop_counter++;
    /* if bit 7 is set, the display is busy, wait and read again */
    if (temp == 0x00) break;		 
    /* done, exit */
    if (loop_counter >= 0xff) {		 
      /* bail out if broken, and the display */
      /* is not ready in 255 reads */
      sprintf(syslog_string_ptr,
	      "Display Error: Busy for 0x%2x attempts. Exiting.\n",
	      loop_counter);			
      /* report the failure */
      syslog_send(syslog_string_ptr);	 
      fflush();
      clean_up(SIGTERM);		 
      /* and go bye-bye */
      exit(-1);
    }
  } while  (temp);
}

void disp_init(unsigned short port_base_addr, unsigned char port_len) {
  /* do the ioperm() call */
  get_port(port_base_addr,port_len);
  /* Reseting Display */
  display_control(0x01);	  
  /* Setting interface parameters */
  display_control(0x38);	 
  /* Enabling display */
  display_control(0x0c);	  
  /* Display (should be) ready */
  /* load the CG ram with custom chars */
  load_cg_ram();
  /* set the pos pointers */
  disp_cp.cur_current_pos = 0x40; 
  remain_chars();
}

void line_2(void) {
  /* set cursor to left side of line 2, char pos 0x40 */
  disp_cp.cur_current_pos = 0x40;
  disp_cp.cur_line = 1;
  remain_chars();
  display_address(0x40);
}

void remain_chars() {
  switch (disp_size) {
  case e_20x2:
    disp_cp.cur_remain_chars = 20;
    break;
  case e_24x2:
    disp_cp.cur_remain_chars = 24;
    break;
  }
}

void display_string(char *pointer) {
  /* display a normal zero terminated string on the LCD display */
  unsigned char write_data;
  do {
    write_data = *pointer;
    if (write_data == 0x00) break;
    
    display_data(write_data);
    pointer++;
  } while(1);
  
  if (disp_cp.auto_fill) {
    do {
      display_data(disp_cp.fill_data);
    } while (disp_cp.cur_remain_chars > -1);
  }
}		

void load_cg_ram(void) { 
  /* generate 5 bar-graph chars */
  int i = 0;
  /* set address of CG ram to 0x00 */
  display_control(0x40); 
  load_eight_bytes(0x1f);
  load_eight_bytes(0x10);
  load_eight_bytes(0x18);
  load_eight_bytes(0x1c);
  load_eight_bytes(0x1e);
  load_eight_bytes(0x1f);	
  /*
   *  And add a Degree symbol.....
   *  54321 (eight bytes total)
   *  01100
   *  10010
   *  10010
   *  01100
   *  00000
   *  00000
   *  00000
   */
  display_data(0x0c);
  display_data(0x12);
  display_data(0x12);
  display_data(0x0c);
  for (i=0; i <= 3; i++) {
    display_data(0x00);	
    /* this is custom char 0x06 */	 
  }
  
  /*
   *  And add an arrow head.....
   *  01000
   *  00100
   *  00010
   *  11111
   *  00010
   *  00100
   *  01000
   *  00000
   */
  display_data(0x08);
  display_data(0x04);
  display_data(0x02);
  display_data(0x1f);
  display_data(0x02);
  display_data(0x04);
  display_data(0x08);
  display_data(0x00);	 
  /* this is custom char 0x07 */
  /* put display back into normal mode */
  display_address(0x00); 
}

void load_eight_bytes(unsigned char data) {
  unsigned char i;				
  /* 5x7 display, 7 bytes of data and a zero... */
  for (i=0; i<7; i++) { 
    display_data(data);
  }
  display_data(0x00);
}

void calc_bar_g_display(float load, int chars) {
  /* calculate how to display the current load (or anything else) */
  /* on the system with a bar graph */
  /* Display will be 'chars' 5x7 chars on the LCD display */
  /* plus a single int numeric char to annotate the max value */
  /* the display auto-scales from 0-1, to 0-(int load+1) */
  /* where  1 <= (int load+1) >= 9 */
  /* Fixed a bug in v0.27 that showed up in the annotation */
  /* for loads greater than 9 */
  /* CW May 30, '98 */
  
  int i = 0;				
  /* convert to an integer */
  bar_g.max = (load + 1);		
  /* store the load in the structure */
  bar_g.load = load;	      		
  bar_g.step_max = bar_g.max;

  if (bar_g.step_max <= 9) {            
    /* Annotation is 0-9, or '^' if >9 */
    bar_g.annot = (bar_g.step_max + '0');
  }
  else {
    bar_g.annot = '^';
  }
  
  /* 5 bars/char in the graph */
  bar_g.step = bar_g.step_max / (5.00 * chars);	
  /* figure out how many bars to display */
  bar_g.bars = load / bar_g.step;	
  /* calculate the number of full chars */
  bar_g.full_chars = bar_g.bars / 5;	
  bar_g.remain_bars = (bar_g.bars - (bar_g.full_chars * 5));
  /* select the correct final char for the graph */
  bar_g.final_char = bar_g.remain_bars;	
  /* build the bar graph string to display */
  if (bar_g.full_chars) {		
    /* only if there is a full block to display */
    for (i = 0; i <= (bar_g.full_chars - 1); i++) {
      bar_g.display_string[i] = 0x05;	
      /* a block of 5 bars */
    }
  }
  
  /* now for the final bar set */
  /* if there are zero remaining bars, we don't display a char */
  if (bar_g.final_char) {
    bar_g.display_string[i] = bar_g.final_char;
    i++;
  }
  
  for (i = i; i <= (chars - 1); i++) {	 
    /* use spaces to fill blank dispaly space */
    bar_g.display_string[i] = 0x20;
  }
  
  /* add the max annotation to the string */
  bar_g.display_string[i] = bar_g.annot; 
  i++;
  /* mark the end of the string */
  bar_g.display_string[i] = 0x00;	 
  /* done */
}					 

