/*
 * 
 * ***** Enter Daemon Mode *****
 * 
 */
int daemon_init(float version) {
  if ((pid = fork()) < 0) return(-1);
  else if (pid != 0) {
    printf("Display Driver Daemon V%1.2f Starting with PID %d\n",
	   version,
	   pid);
    exit(0);
    /* parent goes away here! */
  }
  
  /* become a session leader */
  setsid(); 	   
  /* leave filesystem we are in */
  chdir ("/dev/"); 
  /* clear out file creation mask */
  umask(0); 	   
  /* get our 'real' pid now... */
  pid = getpid();  
  return(pid);
}

/*
 * 	
 * ***** syslog interface functions *****		
 * 
 */
void syslog_temp_stat(void) {
  if (dal_dat.sensor_ok) {
    /* get the temp */
    get_temp(0x00); 
  
    sprintf(syslog_string_ptr,\
	    "Found DS1620. Reporting %3.2f Degrees C\n",
	    dal_dat.degrees_c);
    syslog_send(syslog_string_ptr);	
    
    if (dal_dat.offset != 0) { 
      /* report any offset */
      sprintf(syslog_string_ptr,
	      "Temp. Offset set to %3.2f Degrees C\n",
	      dal_dat.offset);
      syslog_send(syslog_string_ptr);
    }
  }
  
  if (!dal_dat.sensor_ok) {
    sprintf(syslog_string_ptr,
	    "No temp sensor found\n");
    syslog_send(syslog_string_ptr);
  }
}

void syslog_send(unsigned char *syslog_string_ptr) {
  syslog((LOG_DAEMON | LOG_INFO),
	 syslog_string_ptr);
}

void syslog_open(void) {
  openlog(syslog_ident_ptr,
	  LOG_PID,
	  LOG_DAEMON);
}

void alt_temp(void) {
  if (dal_dat.sensor_ok) {
    if (dal_dat.temp_display_type == DEGREES_F){
      sprintf(syslog_string_ptr,
	      "Mode: DEGREES_C %3.2f\n",
	      dal_dat.degrees_c);
    }
    else
      sprintf(syslog_string_ptr,
	      "Mode: DEGREES_F %3.2f\n",
	      dal_dat.degrees_f);
    syslog_send(syslog_string_ptr);
  }
  /* Flip the temp display type bit */
  dal_dat.temp_display_type = 
    ((dal_dat.temp_display_type & 0x1) ^ 0x1);  
}

void log_temp(void) {
  if (dal_dat.sensor_ok) {
    switch (dal_dat.temp_display_type) {
    case DEGREES_F:
      sprintf(syslog_string_ptr,
	      "Log: DEGREES_F %3.2f Max: %3.2f Min: %3.2f\n",
	      dal_dat.degrees_f,
	      dal_dat.max_f,
	      dal_dat.min_f);
      syslog_send(syslog_string_ptr);
      break;

    case DEGREES_C:
      sprintf(syslog_string_ptr,
	      "Log: DEGREES_C %3.2f Max: %3.2f Min: %3.2f\n",
	      dal_dat.degrees_c,
	      dal_dat.max_c,
	      dal_dat.min_c);
      syslog_send(syslog_string_ptr);
      break;
    }
    sprintf(syslog_string_ptr,
	    "Log: Max Time: %s",
	    dal_dat.last_max_time);
    syslog_send(syslog_string_ptr);
    sprintf(syslog_string_ptr,
	    "Log: Min Time: %s",
	    dal_dat.last_min_time);
    syslog_send(syslog_string_ptr);
  }
  else {
    sprintf(syslog_string_ptr,
	    "Log: NO TEMP \n");
    syslog_send(syslog_string_ptr);
  }
}
